<?php

namespace Drupal\cloudinary_media_library_widget\Plugin\Field\FieldWidget;

use Cloudinary\Configuration\Configuration;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\media_library\Plugin\Field\FieldWidget\MediaLibraryWidget;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'cloudinary_media_library_widget' widget.
 *
 * @FieldWidget(
 *   id = "cloudinary_media_library_widget",
 *   label = @Translation("Cloudinary media library"),
 *   description = @Translation("Allows you to select cloudinary assets."),
 *   field_types = {
 *     "entity_reference"
 *   },
 *   multiple_values = TRUE,
 * )
 *
 * @internal
 *   Plugin classes are internal.
 */
class CloudinaryMediaLibraryWidget extends MediaLibraryWidget {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The Cloudinary media manager.
   *
   * @var \Drupal\cloudinary_media_library_widget\CloudinaryMediaManagerInterface
   */
  protected $cloudinaryMediaManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->configFactory = $container->get('config.factory');
    $instance->cloudinaryMediaManager = $container->get('cloudinary_media_library_widget.manager.cloudinary_media');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
        'resource_type' => 'image',
      ] + parent::defaultSettings();
  }

  /**
   * Get resource type options.
   *
   * @return array
   *   List of options.
   */
  protected function getResourceTypeOptions() {
    $options = [];

    foreach ($this->cloudinaryMediaManager->getDefinitions() as $definition) {
      $options[$definition['resource_type']] = $definition['label'];
    }

    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);
    $elements['resource_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Default resource'),
      '#options' => $this->getResourceTypeOptions(),
      '#default_value' => $this->getSetting('resource_type'),
      '#description' => $this->t('Select the default resource type preselected on Cloudinary media widget.'),
      '#required' => TRUE,
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();

    $options = $this->getResourceTypeOptions();
    $summary[] = $this->t('Cloudinary resource type: @resource', [
      '@resource' => $options[$this->getSetting('resource_type')],
    ]);

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    try {
      Configuration::instance()->validate();
    }
    catch (\Exception $e) {
      $this->messenger()->addWarning($this->t('You have not provided a valid connection to your Cloudinary instance. Follow @link to set up a connection.', [
        '@link' => Link::createFromRoute($this->t('this link'), 'cloudinary_sdk.settings')->toString(),
      ]));
    }

    $bundle = current($this->getAllowedMediaTypeIdsSorted());

    $element = parent::formElement($items, $delta, $element, $form, $form_state);
    $element['#attributes']['class'][] = 'js-cloudinary-library-widget';
    $element['#attributes']['data-field-name'] = $this->fieldDefinition->getName();
    $element['media_library_selection']['#attributes']['class'][] = 'js-cloudinary-library-selection';
    $element['media_library_update_widget']['#attributes']['class'][] = 'js-cloudinary-library-update-widget';

    $element['open_button'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['js-cloudinary-library-open-button']],
      '#attached' => [
        'library' => ['cloudinary_media_library_widget/widget'],
        'drupalSettings' => $this->getDrupalJavascriptSettings($bundle),
      ],
    ];

    return $element;
  }

  /**
   * Get drupal javascript settings.
   *
   * @param string $bundle
   *   Media bundle.
   *
   * @return array
   *   Cloudinary js settings.
   */
  protected function getDrupalJavascriptSettings(string $bundle): array {
    $cardinality = $this->fieldDefinition->getFieldStorageDefinition()->getCardinality();

    $sdk_settings = $this->configFactory->get('cloudinary_sdk.settings');
    $widget_settings = $this->configFactory->get('cloudinary_media_library_widget.settings');

    $settings = [
      'cloud_name' => $sdk_settings->get('cloudinary_sdk_cloud_name'),
      'api_key' => $sdk_settings->get('cloudinary_sdk_api_key'),
      'bundle' => $bundle,
      'multiple' => $cardinality != 1,
      'resource_type' => $this->getSetting('resource_type'),
      'use_saml' => $widget_settings->get('cloudinary_saml_auth'),
    ];

    $starting_folder = $widget_settings->get('cloudinary_starting_folder');

    if ($starting_folder !== '/') {
      $settings['starting_folder'] = $starting_folder;
    }

    if ($cardinality != FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED) {
      $settings['max_files'] = $cardinality;
    }

    return [
      'cloudinary_media_library_widget' => [
        $this->fieldDefinition->getName() => $settings,
      ],
    ];
  }

}
