<?php

namespace Drupal\cloudinary_video\Plugin\video_embed_field\Provider;

use Cloudinary\Asset\Media;
use Drupal\Component\Serialization\Json;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Template\Attribute;
use Drupal\video_embed_field\ProviderPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A Cloudinary provider plugin.
 *
 * @VideoEmbedProvider(
 *  id = "cloudinary",
 *  title = @Translation("Cloudinary")
 *  )
 */
class Cloudinary extends ProviderPluginBase {

  /**
   * The config factory interface.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->configFactory = $container->get('config.factory');

    return $instance;
  }

  /**
   * Get the video format.
   *
   * @return string
   *   The video format.
   */
  protected function getVideoFormat() {
    $info = pathinfo($this->getVideoId());

    return $info['extension'];
  }

  /**
   * Get the asset public id.
   *
   * @return string
   *   The asset public id.
   */
  protected function getAssetPublicId() {
    $info = pathinfo($this->getVideoId());

    if ($info['dirname'] === '.') {
      return $info['filename'];
    }

    return $info['dirname'] . '/' . $info['filename'];
  }

  /**
   * Get default video transformations.
   */
  protected function getVideoTransformations() {
    return $this->configFactory
      ->get('cloudinary_video.settings')
      ->get('cloudinary_video_optimizations');
  }

  /**
   * Get default image transformations.
   */
  protected function getImageTransformations() {
    return $this->configFactory
      ->get('cloudinary_media_library_widget.settings')
      ->get('cloudinary_image_optimizations');
  }

  /**
   * {@inheritdoc}
   */
  public function downloadThumbnail() {
    $local_uri = $this->getLocalThumbnailUri();
    if (!file_exists($local_uri)) {
      $directory = $this->thumbsDirectory;
      $info = pathinfo($this->getVideoId());

      if ($info['dirname'] !== '.') {
        $directory .= "/{$info['dirname']}";
      }

      $this->getFileSystem()->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS);
      try {
        $thumbnail = $this->httpClient->request('GET', $this->getRemoteThumbnailUrl());
        $this->getFileSystem()->saveData((string) $thumbnail->getBody(), $local_uri);
      }
      catch (\Exception $e) {
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getRemoteThumbnailUrl() {
    $source = str_replace(".{$this->getVideoFormat()}", '.jpg', $this->getVideoId());
    $options['secure'] = TRUE;
    $options['resource_type'] = 'video';

    if ($transformation = $this->getImageTransformations()) {
      $options['raw_transformation'] = $transformation;
    }

    return Media::fromParams($source, $options)->toUrl();
  }

  /**
   * Convert video uri to the absolute path.
   *
   * @return string
   *   The url to use for remote video.
   */
  public function getRemoteVideoUrl() {
    $source = $this->getAssetPublicId();

    $options['secure'] = TRUE;
    $options['resource_type'] = 'video';

    if ($transformation = $this->getVideoTransformations()) {
      $options['raw_transformation'] = $transformation;
    }

    return Media::fromParams($source, $options)->toUrl();
  }

  /**
   * Get cloud name from the SDK settings.
   *
   * @return string
   *   The cloud name.
   */
  protected function getCloudName() {
    return $this->configFactory->get('cloudinary_sdk.settings')
      ->get('cloudinary_sdk_cloud_name');
  }

  /**
   * Get CNAME from the SDK settings.
   *
   * @return string
   *   The CNAME.
   */
  protected function getCname() {
    return $this->configFactory->get('cloudinary_sdk.settings')
      ->get('cloudinary_sdk_cname');
  }

  /**
   * Get video player configuration.
   *
   * @return string
   *   The encoded json to be used for video player configuration.
   */
  protected function getVideoPlayerConfiguration() {
    $config_key = $this->getVideoPlayerType() === 'html5'
      ? 'cloudinary_default_player_config'
      : 'cloudinary_video_player_config';

    return $this->configFactory->get('cloudinary_video.settings')
      ->get($config_key);
  }

  /**
   * Render embed cloudinary player.
   *
   * @param string $width
   *   Default width of the widget.
   * @param string $height
   *   Default height of the widget.
   * @param bool $autoplay
   *   Whether to autoplay video or not.
   *
   * @return array
   *   Renderable array with video player.
   */
  protected function renderCloudinaryPlayer($width, $height, $autoplay) {
    $embed_code = [
      '#type' => 'video_embed_iframe',
      '#provider' => 'cloudinary',
      '#url' => 'https://player.cloudinary.com/embed/',
      '#query' => [
        'cloud_name' => $this->getCloudName(),
        'public_id' => $this->getAssetPublicId(),
      ],
      '#attributes' => [
        'width' => $width,
        'frameborder' => '0',
        'allow' => 'autoplay; fullscreen; encrypted-media; picture-in-picture',
        'allowfullscreen' => 'true',
      ],
      '#cache' => [
        'tags' => ['config:cloudinary_video.settings'],
      ],
    ];

    // Support custom domain.
    if ($cname = $this->getCname()) {
      $embed_code['#query']['cloudinary'] = [
        'secure_distribution' => $cname,
        'private_cdn' => TRUE,
      ];
    }

    if ($settings = $this->getVideoPlayerConfiguration()) {
      $player = Json::decode($settings);
      $embed_code['#query']['player'] = $player;

      if ($autoplay) {
        $embed_code['#query']['player']['autoplay'] = 'true';
        $embed_code['#query']['player']['autoplayMode'] = 'always';
      }
    }

    if ($transformation = $this->getVideoTransformations()) {
      $embed_code['#query']['source']['transformation']['raw_transformation'] = $transformation;
    }

    return $embed_code;
  }

  /**
   * Render default html5 player.
   *
   * @param string $width
   *   Default width of the widget.
   * @param string $height
   *   Default height of the widget.
   * @param bool $autoplay
   *   Whether to autoplay video or not.
   *
   * @return array
   *   Renderable array with video player.
   */
  protected function renderDefaultPlayer($width, $height, $autoplay) {
    $attributes = new Attribute([
      'width' => '100%',
      'controls' => 'controls',
    ]);

    if ($autoplay) {
      $attributes->setAttribute('autoplay', 'autoplay');
    }

    if ($player_attributes = $this->getVideoPlayerConfiguration()) {
      foreach (explode(' ', $player_attributes) as $attribute) {
        $data = explode('=', $attribute);
        $attributes->setAttribute($data[0], $data[1] ?? $data[0]);
      }
    }

    return [
      '#type' => 'inline_template',
      '#template' => '<video{{ attributes }}><source src="{{ url }}" /></video>',
      '#context' => [
        'url' => $this->getRemoteVideoUrl(),
        'attributes' => $attributes,
        'public_id' => $this->getAssetPublicId(),
      ],
      '#cache' => [
        'tags' => ['config:cloudinary_video.settings'],
      ],
    ];
  }

  /**
   * Get video player type.
   *
   * @return string
   *   The video player type.
   */
  protected function getVideoPlayerType() {
    return $this->configFactory->get('cloudinary_video.settings')
      ->get('cloudinary_default_player');
  }

  /**
   * {@inheritdoc}
   */
  public function renderEmbedCode($width, $height, $autoplay) {
    switch ($this->getVideoPlayerType()) {
      case 'cloudinary':
        return $this->renderCloudinaryPlayer($width, $height, $autoplay);

      case 'html5':
        return $this->renderDefaultPlayer($width, $height, $autoplay);
    }

    return [];
  }

  /**
   * {@inheritdoc}
   */
  public static function getIdFromInput($input) {
    // Get public id from the video schema.
    if (str_contains($input, 'cloudinary://')) {
      return substr($input, strlen('cloudinary://'));
    }

    if (str_contains($input, '/video/upload/')) {
      \Drupal::messenger()->addWarning(t('External cloudinary URL is currently not supported.'));
    }

    return FALSE;
  }

}
